Package: nullranges
Title: Generation of null ranges via bootstrapping or covariate
        matching
Version: 1.18.0
Authors@R: c(
    person("Michael", "Love", role = c("aut", "cre"),
      email = "michaelisaiahlove@gmail.com",
      comment = c(ORCID = "0000-0001-8401-0545")),
    person("Wancen", "Mu", role = c("aut"),
      comment = c(ORCID = "0000-0002-5061-7581")),
    person("Eric", "Davis", role = c("aut"),
      comment = c(ORCID = "0000-0003-4051-3217")),
    person("Douglas", "Phanstiel", role = c("aut"),
      comment = c(ORCID = "0000-0003-2123-0051")),
    person("Stuart", "Lee", role = c("aut"),
      comment = c(ORCID = "0000-0003-1179-8436")),
    person("Mikhail", "Dozmorov", role = c("ctb")),
    person("Tim", "Triche", role = c("ctb")),
    person("CZI", role = c("fnd"))
    )
Description: Modular package for generation of sets of ranges
        representing the null hypothesis. These can take the form of
        bootstrap samples of ranges (using the block bootstrap
        framework of Bickel et al 2010), or sets of control ranges that
        are matched across one or more covariates. nullranges is
        designed to be inter-operable with other packages for analysis
        of genomic overlap enrichment, including the plyranges
        Bioconductor package.
Suggests: testthat, knitr, rmarkdown, ks, DNAcopy, RcppHMM,
        AnnotationHub, ExperimentHub, GenomeInfoDb, nullrangesData,
        ensembldb, EnsDb.Hsapiens.v86, BSgenome.Hsapiens.UCSC.hg38,
        patchwork, plotgardener, dplyr, magrittr, tidyr, cobalt,
        DiagrammeR, MatchIt, mariner
Imports: stats, IRanges, GenomicRanges, Seqinfo, methods, rlang,
        S4Vectors, scales, InteractionSet, ggplot2, grDevices,
        plyranges, data.table, progress, ggridges
biocViews: Visualization, GeneSetEnrichment, FunctionalGenomics,
        Epigenetics, GeneRegulation, GeneTarget, GenomeAnnotation,
        Annotation, GenomeWideAssociation, HistoneModification,
        ChIPSeq, ATACSeq, DNaseSeq, RNASeq, HiddenMarkovModel
URL: https://nullranges.github.io/nullranges,
        https://github.com/nullranges/nullranges
BugReports: https://support.bioconductor.org/tag/nullranges/
License: GPL-3
Encoding: UTF-8
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Config/pak/sysreqs: make libbz2-dev liblzma-dev libxml2-dev libssl-dev
        xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:57:11 UTC
RemoteUrl: https://github.com/bioc/nullranges
RemoteRef: RELEASE_3_23
RemoteSha: b4dd7bf8a7d8748290f066b47ade55809bbbc341
NeedsCompilation: no
Packaged: 2026-05-01 09:44:02 UTC; root
Author: Michael Love [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8401-0545>),
  Wancen Mu [aut] (ORCID: <https://orcid.org/0000-0002-5061-7581>),
  Eric Davis [aut] (ORCID: <https://orcid.org/0000-0003-4051-3217>),
  Douglas Phanstiel [aut] (ORCID:
    <https://orcid.org/0000-0003-2123-0051>),
  Stuart Lee [aut] (ORCID: <https://orcid.org/0000-0003-1179-8436>),
  Mikhail Dozmorov [ctb],
  Tim Triche [ctb],
  CZI [fnd]
Maintainer: Michael Love <michaelisaiahlove@gmail.com>
Depends: R (>= 4.2.0)
Built: R 4.6.0; ; 2026-05-01 10:05:43 UTC; unix
