# Generated by roxygen2: do not edit by hand

S3method(names,MatchedOverview)
export(BootRanges)
export(Matched)
export(MatchedDataFrame)
export(MatchedGInteractions)
export(MatchedGRanges)
export(bootRanges)
export(combnCov)
export(covariates)
export(focal)
export(indices)
export(makeExampleMatchedDataSet)
export(matchRanges)
export(matched)
export(matchedData)
export(matchitToMatched)
export(method)
export(oneRegionSegment)
export(overview)
export(plotCovariate)
export(plotPropensity)
export(plotSegment)
export(pool)
export(reduceSegment)
export(segmentDensity)
export(unmatched)
export(withReplacement)
exportClasses(BootRanges)
exportClasses(Matched)
exportClasses(MatchedDataFrame)
exportClasses(MatchedGInteractions)
exportClasses(MatchedGRanges)
exportMethods("$")
exportMethods("[[")
exportMethods(combnCov)
exportMethods(covariates)
exportMethods(focal)
exportMethods(indices)
exportMethods(makeExampleMatchedDataSet)
exportMethods(matchRanges)
exportMethods(matched)
exportMethods(matchedData)
exportMethods(matchitToMatched)
exportMethods(method)
exportMethods(overview)
exportMethods(plotCovariate)
exportMethods(plotPropensity)
exportMethods(pool)
exportMethods(show)
exportMethods(unmatched)
exportMethods(withReplacement)
import(GenomicRanges)
import(InteractionSet)
import(S4Vectors)
import(data.table, except = c(between, shift, first, second, indices))
import(ggplot2)
import(ggridges)
import(methods)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,sort)
importFrom(GenomicRanges,tileGenome)
importFrom(IRanges,IRanges)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,mid)
importFrom(IRanges,overlapsAny)
importFrom(IRanges,successiveIRanges)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Rle)
importFrom(Seqinfo,"seqlengths<-")
importFrom(Seqinfo,genome)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,sortSeqlevels)
importFrom(data.table,data.table)
importFrom(grDevices,palette.colors)
importFrom(plyranges,as_granges)
importFrom(plyranges,join_overlap_intersect)
importFrom(rlang,"!!")
importFrom(rlang,enquo)
importFrom(rlang,f_lhs)
importFrom(rlang,f_rhs)
importFrom(scales,breaks_extended)
importFrom(scales,label_comma)
importFrom(scales,squish_infinite)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,glm)
importFrom(stats,kmeans)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,terms)
